@@ -2,11 +2,16 @@ package helpers
 
 import (
 	"errors"
+	"io"
+	"io/ioutil"
 	"os"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
+	mock "github.com/stretchr/testify/mock"
+	"github.com/stretchr/testify/require"
 
+	"gitlab.com/gitlab-org/gitlab-runner/commands/helpers/archive"
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers"
 )
@@ -307,6 +312,48 @@ func TestArtifactsExcludedPaths(t *testing.T) {
 	assert.Equal(t, 1, network.uploadCalled)
 }
 
+func TestFileArchiverCompressionLevel(t *testing.T) {
+	writeTestFile(t, artifactsTestArchivedFile)
+	defer os.Remove(artifactsTestArchivedFile)
+
+	network := &testNetwork{
+		uploadState: common.UploadSucceeded,
+	}
+
+	for _, expectedLevel := range []string{"fastest", "fast", "default", "slow", "slowest"} {
+		t.Run(expectedLevel, func(t *testing.T) {
+			mockArchiver := new(archive.MockArchiver)
+			defer mockArchiver.AssertExpectations(t)
+
+			archive.Register(
+				"zip",
+				func(w io.Writer, dir string, level archive.CompressionLevel) (archive.Archiver, error) {
+					assert.Equal(t, getCompressionLevel(expectedLevel), level)
+					return mockArchiver, nil
+				},
+				nil,
+			)
+
+			mockArchiver.On("Archive", mock.Anything, mock.Anything).Return(nil)
+
+			cmd := ArtifactsUploaderCommand{
+				JobCredentials: UploaderCredentials,
+				network:        network,
+				Format:         common.ArtifactFormatZip,
+				fileArchiver: fileArchiver{
+					Paths: []string{artifactsTestArchivedFile},
+				},
+				CompressionLevel: expectedLevel,
+			}
+			assert.NoError(t, cmd.enumerate())
+			_, r, err := cmd.createReadStream()
+			require.NoError(t, err)
+			defer r.Close()
+			_, _ = io.Copy(ioutil.Discard, r)
+		})
+	}
+}
+
 func TestArtifactUploaderCommandShouldRetry(t *testing.T) {
 	tests := map[string]struct {
 		err   error
