@@ -31,6 +31,7 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/test"
+	"gitlab.com/gitlab-org/gitlab-runner/shells"
 )
 
 var getWindowsImageOnce sync.Once
@@ -465,6 +466,12 @@ func TestDockerCommandBuildCancel(t *testing.T) {
 	buildtest.RunBuildWithCancel(t, getRunnerConfigForOS(t), nil)
 }
 
+func TestBuildMasking(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	buildtest.RunBuildWithMasking(t, getRunnerConfigForOS(t), nil)
+}
+
 func TestDockerCommandTwoServicesFromOneImage(t *testing.T) {
 	test.SkipIfGitLabCIOn(t, test.OSWindows)
 	helpers.SkipIntegrationTests(t, "docker", "info")
@@ -1213,6 +1220,31 @@ func TestDockerCommandWithHelperImageConfig(t *testing.T) {
 	)
 }
 
+func TestDockerCommand_Pwsh(t *testing.T) {
+	test.SkipIfGitLabCIOn(t, test.OSWindows)
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	build := getBuildForOS(t, common.GetRemoteSuccessfulBuild)
+	build.Image.Name = common.TestPwshImage
+	build.Runner.Shell = shells.SNPwsh
+	build.JobResponse.Steps = common.Steps{
+		common.Step{
+			Name: common.StepNameScript,
+			Script: []string{
+				"Write-Output $PSVersionTable",
+			},
+			Timeout:      120,
+			When:         common.StepWhenAlways,
+			AllowFailure: false,
+		},
+	}
+
+	out, err := buildtest.RunBuildReturningOutput(t, &build)
+	assert.NoError(t, err)
+	assert.Contains(t, out, "PSVersion                      7.1.1")
+	assert.Contains(t, out, "PSEdition                      Core")
+}
+
 func TestDockerCommandWithDoingPruneAndAfterScript(t *testing.T) {
 	test.SkipIfGitLabCIOn(t, test.OSWindows)
 	helpers.SkipIntegrationTests(t, "docker", "info")
@@ -1534,3 +1566,9 @@ func TestChownAndUmaskUsage(t *testing.T) {
 		})
 	}
 }
+
+func TestBuildLogLimitExceeded(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	buildtest.RunRemoteBuildWithJobOutputLimitExceeded(t, getRunnerConfigForOS(t), nil)
+}
