@@ -14,11 +14,7 @@ func Test_windowsInfo_create(t *testing.T) {
 	revision := "4011f186"
 
 	for _, shell := range []string{"", "powershell", "pwsh"} {
-		expectedPowershellCmdLine := []string{shell}
-		if shell == "" {
-			expectedPowershellCmdLine[0] = "powershell"
-		}
-		expectedPowershellCmdLine = append(expectedPowershellCmdLine, powerShellCmdArgs...)
+		expectedPowershellCmdLine := getPowerShellCmd(shell)
 
 		tests := []struct {
 			operatingSystem string
