@@ -1,7 +1,6 @@
 package secrets
 
 import (
-	"errors"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
@@ -13,13 +12,11 @@ func TestResolvingUnsupportedSecretError_Error(t *testing.T) {
 }
 
 func TestResolvingUnsupportedSecretError_Is(t *testing.T) {
-	assert.True(
+	assert.ErrorIs(
 		t,
-		errors.Is(
-			NewResolvingUnsupportedSecretError("expected"),
-			NewResolvingUnsupportedSecretError("expected"),
-		),
+		NewResolvingUnsupportedSecretError("expected"),
+		NewResolvingUnsupportedSecretError("expected"),
 	)
-	assert.False(t, errors.Is(NewResolvingUnsupportedSecretError("expected"), new(ResolvingUnsupportedSecretError)))
-	assert.False(t, errors.Is(NewResolvingUnsupportedSecretError("expected"), assert.AnError))
+	assert.NotErrorIs(t, NewResolvingUnsupportedSecretError("expected"), new(ResolvingUnsupportedSecretError))
+	assert.NotErrorIs(t, NewResolvingUnsupportedSecretError("expected"), assert.AnError)
 }
