@@ -30,6 +30,7 @@ func (b *AbstractShell) GetFeatures(features *common.FeaturesInfo) {
 	features.ArtifactsExclude = true
 	features.MultiBuildSteps = true
 	features.VaultSecrets = true
+	features.ReturnExitCode = true
 }
 
 func (b *AbstractShell) writeCdBuildDir(w ShellWriter, info common.ShellScriptInfo) {
@@ -353,7 +354,10 @@ func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, build *common.Build,
 	w.Command("git", "remote", "set-url", "origin", build.GetRemoteURL())
 	w.EndIf()
 
-	fetchArgs := []string{"fetch", "origin"}
+	v := common.AppVersion
+	userAgent := fmt.Sprintf("http.userAgent=%s %s %s/%s", v.Name, v.Version, v.OS, v.Architecture)
+
+	fetchArgs := []string{"-c", userAgent, "fetch", "origin"}
 	fetchArgs = append(fetchArgs, build.GitInfo.Refspecs...)
 	if depth > 0 {
 		fetchArgs = append(fetchArgs, "--depth", strconv.Itoa(depth))
